import subprocess


def reboot_system():
    subprocess.Popen(['/sbin/shutdown', '-r', 'now'])


def kexec_system():
    proc = subprocess.Popen(['grubby','--default-kernel'],stdout=subprocess.PIPE)
    kernel = proc.stdout.readline().rstrip()
    infoargs = "--info=" + kernel

    proc = subprocess.Popen(['grubby',infoargs],stdout=subprocess.PIPE)
    for line in proc.stdout:
        if line.rstrip().startswith('args'):
            commandline = line.rstrip().split('"')[1]
        if line.rstrip().startswith('root'):
            commandline = line.rstrip() + " " + commandline
        if line.rstrip().startswith('initrd'):
            commandinitrd = line.rstrip()

    proc = subprocess.Popen(['mokutil','--sb-state'],stdout=subprocess.PIPE)
    sb_flag = ""
    if "SecureBoot enabled" in proc.stdout.readline().rstrip():
        sb_flag = "-s"

    cmd = 'kexec {} -l {} --{} --command-line="{}"'.format(sb_flag,kernel,commandinitrd,commandline)

    process = subprocess.Popen('/bin/bash', stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    out, err = process.communicate(cmd)

    cmd = 'kexec -e'
    process = subprocess.Popen('/bin/bash', stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    out, err = process.communicate(cmd)


def get_api_models(actor, what):
    """
    Used to retrieve the full list of models including the ones defined by WorkflowAPIs used by the actor.

    :param what: A string which either is 'consumes' or 'produces'
    :type what: str
    :param actor: Actor type/instance or ActorDefinition instance to retrieve the information from
    :type actor: Actor or ActorDefinition
    :return: Tuple of all produced or consumed models as specified by actor and APIs used by the actor.
    """

    def _enforce_tuple(x):
        if not isinstance(x, tuple):
            return (x,)
        return x

    def _do_get(api):
        result = _enforce_tuple(getattr(api, what, ()))
        for a in _enforce_tuple(api.apis or ()):
            result = result + _do_get(a)
        return result
    return tuple(set(_do_get(actor)))
